import os
import random
import sys

MAX_N = 500
MAX_M = 50000

CASE = 7
DEST = os.path.join(os.path.dirname(__file__), 'secret')
SHUFFLE_DATA = True

def next_file(short_desc=None, long_desc=None):
    global CASE
    basename = os.path.join(DEST, '%02d' % CASE)
    CASE += 1
    if short_desc is not None:
        basename += '-' + short_desc
    if long_desc is not None:
        with open(basename+'.desc', 'w') as desc_out:
            desc_out.write(long_desc)
    return open(basename+'.in', 'w')

def save_case(n, journeys, short_desc=None, long_desc=None):
    sys.stderr.write('save_case n %d m %d shortdesc %s\n' % (n, len(journeys), short_desc))
    f = next_file(short_desc, long_desc)
    f.write('%d %d\n' % (n, len(journeys)))
    if SHUFFLE_DATA:
        random.shuffle(journeys)
    for (a, b) in journeys:
        if SHUFFLE_DATA and random.randint(0, 1): (a, b) = (b, a)
        f.write('%d %d\n' % (a, b))
    f.close()


def gen_random(n, m):
    journeys = [random.sample(range(1, n+1), 2) for _ in range(m)]
    save_case(n, journeys, short_desc='random-n%d-m%d' % (n, m))

def gen_random_far(n, m, d):
    possible = [(i, j) for i in range(1, n+1) for j in range(i+d, n+1)]
    journeys = [random.choice(possible) for _ in range(m)]
    save_case(n, journeys, short_desc='random-far-n%d-m%d-d%d' % (n, m, d))

def gen_random_close(n, m, d):
    possible = [(i, j) for i in range(1, n+1) for j in range(i+1, min(n, i+d)+1)]
    journeys = [random.choice(possible) for _ in range(m)]
    save_case(n, journeys, short_desc='random-close-n%d-m%d-d%d' % (n, m, d))


def gen_complete(n):
    save_case(n, [(i+1, j+1) for i in range(n) for j in range(i)], short_desc='complete-n%d' % (n))


def gen_all_from_first(n, m):
    save_case(n, [(1, random.randint(2, n)) for _ in range(m)], short_desc='all-from-first')

def gen_all_from_last(n, m):
    save_case(n, [(n, random.randint(1, n-1)) for _ in range(m)], short_desc='all-from-last')

def gen_crossing(n, m):
    c = random.randint(1, n)
    journeys = []
    while len(journeys) < m:
        journeys.append((random.randint(1, c), random.randint(c, n)))
        if journeys[-1] == (c, c): journeys.pop()
    save_case(n, journeys, short_desc='crossing', long_desc='All journeys are between x and y with x <= %d and y >= %d\n' % (c, c))

def gen_menger(n, m):
    journeys = []
    def rec(x0, x1, y0, y1, m):
        assert x0 < x1 and y0 < y1 and m > 0, (x0, x1, y0, y1, m)
        if x1-x0 <= 2 or y1-y0 <= 2 or m < 8:
            for _ in range(m):
                while True:
                    x = random.randint(x0, x1-1)
                    y = random.randint(y0, y1-1)
                    if x != y:
                        journeys.append((x+1, y+1))
                        break
            return
        extra = [1]*(m % 8) + [0]*(9-m%8)
        while extra[4]:
            random.shuffle(extra)
        for dx in range(3):
            for dy in range(3):
                if dx == 1 and dy == 1: continue
                rec(((3-dx)*x0 + dx*x1)/3, ((2-dx)*x0 + (1+dx)*x1)/3,
                    ((3-dy)*y0 + dy*y1)/3, ((2-dy)*y0 + (1+dy)*y1)/3,
                    m / 8 + extra[3*dy+dx])
    rec(0, n, 0, n, m)
    assert len(journeys) == m
    save_case(n, journeys, short_desc='menger')

def gen_sierpinski(n, m):
    journeys = []
    def rec(x0, x1, y0, y1, m):
        assert x0 < x1 and y0 < y1 and m > 0, (x0, x1, y0, y1, m)
        if x1-x0 <= 1 or y1-y0 <= 1 or m < 3:
            if x1 - x0 == 1 and y1 - y0 == 1 and x0 == y0: return
            for _ in range(m):
                while True:
                    x = random.randint(x0, x1-1)
                    y = random.randint(y0, y1-1)
                    if x != y:
                        journeys.append((x+1, y+1))
                        break
            return
        extra = [1]*(m % 3) + [0]*(3-m%3)
        random.shuffle(extra)
        rec(x0, (x0+x1)/2, y0, (y0+y1)/2, m/3 + extra[0])
        rec((x0+x1)/2, x1, y0, (y0+y1)/2, m/3 + extra[1])
        rec((x0+x1)/2, x1, (y0+y1)/2, y1, m/3 + extra[2])
    rec(1, n, 0, n-1, m)
    save_case(n, journeys, short_desc='sierpinski')


def main():
    random.seed(23)

    save_case(2, [(2, 1)], short_desc='minimal')
    save_case(MAX_N, [(23, 42)], short_desc='singlejourney')

    gen_complete(10)
    n = 1
    while n*(n+1)/2 <= MAX_M: n += 1
    gen_complete(n)

    gen_random(MAX_N, MAX_M)

    for _ in range(10):
        n = random.randint(2, MAX_N)
        m = random.randint(1, MAX_M)
        gen_random(n, m)

    gen_all_from_first(MAX_N, MAX_M)
    gen_all_from_last(MAX_N, MAX_M)
    gen_crossing(MAX_N, MAX_M)

    gen_random_far(MAX_N, MAX_M, MAX_N/10)
    for _ in range(5):
        n = random.randint(2, MAX_N)
        m = random.randint(1, MAX_M)
        d = random.randint(1, n-1)
        gen_random_far(n, m, d)

    gen_random_close(MAX_N, MAX_M, 1)
    for _ in range(5):
        n = random.randint(2, MAX_N)
        m = random.randint(1, MAX_M)
        d = random.randint(1, n/2)
        gen_random_close(n, m, d)

    gen_menger(MAX_N, MAX_M)
    gen_sierpinski(MAX_N, MAX_M)

if __name__=='__main__':
    main()
