#include <vector>
#include <tuple>
#include <cmath>
#include "validate.h"

using namespace std;

int n;
vector<vector<int>> journeys;

void read_input(istream &in) {
    int t;
    in >> n >> t;
    journeys = vector<vector<int>>(n+1, vector<int>(n+1, 0));
    for (int i = 0; i < t; ++i) {
        int a, b;
        in >> a >> b;
        if (a > b) swap(a, b);
        ++journeys[a][b];
    }
}

// Reads a claimed solution (and checks formatting of the input stream)
pair<int, vector<int>> read_solution(istream &sol, feedback_function feedback) {
    int cost;
    if (!(sol >> cost))
        feedback("Failed to read claimed cost");
    vector<int> H(n+1);
    for (int i = 1; i <= n; ++i) {
        if (!(sol >> H[i]))
            feedback("Failed to read height of %d'th tree", i);
        if (H[i] < 1 || H[i] > 1000000000)
            feedback("Height %d is %d, outside allowed range", i, H[i]);
    }

    string trailing;
    if (sol >> trailing)
        feedback("Trailing output");

    return make_pair(cost, H);
}

// Compute cost of a given height sequence
int compute_cost(const vector<int> &H) {
    vector<int> nextH(n+1), prevH(n+1); // next/prev pos that is larger or equal
    for (int i = 1; i <= n; ++i) {
        for (int j = i+1; j <= n; ++j)
            if (H[j] >= H[i]) {
                nextH[i] = j;
                break;
            }
        for (int j = i-1; j >= 1; --j)
            if (H[j] >= H[i]) {
                prevH[i] = j;
                break;
            }
    }
    // Compute costs for all journeys
    int res = 0;
    for (int l = 1; l <= n; ++l) {
        for (int r = 1; r <= n; ++r) {
            int a = l, b = r, cost = 0;
            while (a < b) {
                if (H[a] < H[b]) a = nextH[a];
                else b = prevH[b];
                ++cost;
            }
            res += cost * journeys[l][r];
        }
    }
    return res;
}


int main(int argc, char **argv) {
    init_io(argc, argv);

    read_input(judge_in);

    int ClaimedOutCost, ClaimedAnsCost;
    vector<int> OutH, AnsH;

    tie(ClaimedOutCost, OutH) = read_solution(cin, wrong_answer);
    tie(ClaimedAnsCost, AnsH) = read_solution(judge_ans, judge_error);

    int ActualOutCost = compute_cost(OutH);
    int ActualAnsCost = compute_cost(AnsH);

    if (ClaimedOutCost != ActualOutCost)
        wrong_answer("Output claims solution has cost %d but its cost is actually %d", ClaimedOutCost, ActualOutCost);

    if (ClaimedAnsCost != ActualAnsCost)
        judge_error("Answer claims solution has cost %d but its cost is actually %d", ClaimedAnsCost, ActualAnsCost);

    if (ActualOutCost < ActualAnsCost)
        judge_error("Output has smaller cost %d than cost of answer file %d", ActualOutCost, ActualAnsCost);

    if (ActualAnsCost < ActualOutCost)
        wrong_answer("Suboptimal cost %d (opt = %d)", ActualAnsCost, ActualOutCost);

    assert(ActualAnsCost == ActualOutCost);

    accept();
}
