#!/usr/bin/env python

"""
run to create the figure on the problem statement.
the input used is ../data/sample/2.{in,ans}
"""

import random
test_case = '../data/sample/2'

if __name__ == '__main__':
  paths = [map(lambda x: int(x)-1, i.split()) for i in open('%s.in' % (test_case), 'r').readlines()[1:]]
  trees = map(int, open('%s.ans' % (test_case), 'r').readlines()[1].split())
  n = len(trees)
  paths = [(i, j) for j in range(n) for i in range(j)]
  drawn_paths = [(1, 7)]

  s = [
    '\\documentclass[class=article,border=0pt]{standalone}\n',
    '\\usepackage[usenames,dvipsnames]{xcolor}\n',
    '\\usepackage{tikz}\n',
    '\\usepackage{xcolor}\n',
    '\\definecolor{lightgreen}{rgb}{0.5,1.0,0.5}\n',
    '\\definecolor{lightbrown}{rgb}{0.87,0.72,0.53}\n',
    '\\begin{document}%\n',
    '\\pagestyle{empty}%\n',
    '\\thispagestyle{empty}%\n',
    '\\begin{tikzpicture}[scale=0.25]%\n']
  y0 = 0
  k = 0

  # Draw the ground.
  s.append('\\draw [draw=black, fill=lightbrown] ({0},{1}) to ({0},{2}) to ({3},{4}) to ({3},{5});\n'.format(
    2, -3, 0, 10*n-2, 0, -3))

  # Draw the bridges.
  bridges = set()
  for a,b in map(sorted, paths):
    while a <> b:
      c = b
      for j in range(b-1,a,-1):
        if trees[j] >= min(trees[c], trees[a]):
          c = j
      bridges.add((a,c,min(trees[a],trees[c]),(a==n/2-2 and c==n/2-1)))
      a = c

  def bridge_vertices(a, b, height):
    st = (10*a+5, 5*height-0.25)
    en = (10*b+5, 5*height-0.25)
    magnitude = abs(en[0]-st[0])**0.75 / 2.0
    def y(x):
      return (x-0)*(x-1)*magnitude+en[1]

    width = 10 * abs(a-b)
    return [st] + [(st[0]+(en[0]-st[0])*i/float(width), y(i/float(width))) for i in range(1,width)] + [en]

  for a,b,height,has_orangutan in bridges:
    xy = bridge_vertices(a, b, height)
    s.append('\\draw [draw=black!50,thick] {0};\n'.format(' to '.join(map(str, xy))))
    s.append('\\draw [draw=black!40] {0};\n'.format(' to '.join(map(lambda i: str((i[0], i[1]+0.75)), xy))))
    for i in xy:
      s.append('\\draw [draw=black!20] ({0},{1}) to ({2},{3});\n'.format(i[0],i[1],i[0],i[1]+0.75))
    if has_orangutan:
      cx = xy[len(xy)/2][0]
      cy = xy[len(xy)/2][1]
      s.append('\\node[align=center] at ({0},{1}) {{\\includegraphics[height=0.85cm]{{louie}}}};'.format(cx, cy-0.8))

  # Draw the trees.
  for i,height in enumerate(trees):
    x = 10*i+5
    pts = [(1, 0), (1, 2)]
    for j in range(height):
      pts += [(2, j*5+1.5)]
      if j + 1 < height:
        pts += [(1, j*5+7)]
      else:
        pts += [(0, j*5+7)]
    s.append('\\node[align=center] at ({0},{1}) {{\Large ${2}$}};'.format(x, -1.5, i+1))
    s.append('\\draw [draw=black,fill=lightgreen] ({0},{1})\n'.format(x+pts[0][0], pts[0][1]))
    for a,b in (pts + [(-ab[0],ab[1]) for ab in pts[::-1]]):
      s.append(' to ({0},{1})'.format(x+a,b))
    s.append(';\n')

  # Calculate the route for paths to draw.
  bold_paths = []
  for path in drawn_paths:
    (a, b) = path
    bold_paths += [[(a, 0)]]
    while a <> b:
      c = a + 1
      for x in range(a+2, b+1):
        if c and trees[c] < min(trees[a], trees[x]):
          c = x
      bridge_height = min(trees[a], trees[c])
      bold_paths[-1] += [(a, bridge_height), (c, bridge_height)]
      a = c
    bold_paths[-1] += [(b, 0)]

  # Draw the highlighted paths.
  for bold_path in bold_paths:
    path = []
    for a,b in zip(bold_path[:-1], bold_path[1:]):
      if a[0] != b[0]:
        path += bridge_vertices(a[0], b[0], a[1])
      elif a[1] != b[1]:
        path += [(10*a[0]+5, 5*a[1]-0.25), (10*a[0]+5, 5*b[1]-0.25)]
    s.append('\\draw [draw=red,thick] {0};\n'.format(' to '.join(map(str, path))))

  s.append('\\end{tikzpicture}\n')
  s.append('\\end{document}\n')
  f = open('fig.tex', 'w')
  for x in s:
    f.write(x)
  f.close()
